/***
*
* File.......: FileLog.ch
*
* Author.....: John Van Ostrand
*
* Description: Translates the USE, CLOSE, SET INDEX TO, and INDEX ON...TO
*              commands to include file logging.
*
* Notes......: For use with Nantucket's Clipper v5.01
*
***/

#command USE                            => dbCloseArea() ;              ;
                                           DbfCloseLog()

#command USE <(db)>                                                     ;
             [VIA <rdd>]                                                ;
             [ALIAS <a>]                                                ;
             [<new: NEW>]                                               ;
             [<ex: EXCLUSIVE>]                                          ;
             [<sh: SHARED>]                                             ;
             [<ro: READONLY>]                                           ;
             [INDEX <(index1)> [, <(indexn)>]]                          ;
                                                                        ;
      => dbUseArea(                                                     ;
                    <.new.>, <rdd>, <(db)>, <(a)>,                      ;
                    if(<.sh.> .or. <.ex.>, !<.ex.>, NIL), <.ro.>        ;
                  ) ;                                                   ;
         DbfLog( <(db)> )                                               ;
                                                                        ;
      [; dbSetIndex( <(index1)> ) ; NtxLog( <(index1)> )]               ;
      [; dbSetIndex( <(indexn)> ) ; NtxLog( <(indexn)> )]



#command SET INDEX TO [ <(index1)> [, <(indexn)>]]                      ;
                                                                        ;
      => dbClearIndex() ;                                               ;
         NtxCloseLog()                                                  ;
                                                                        ;
      [; dbSetIndex( <(index1)> ) ; NtxLog( <(index1)> )]               ;
      [; dbSetIndex( <(indexn)> ) ; NtxLog( <(indexn)> )]


#command INDEX ON <key> TO <(file)> [<u: UNIQUE>]                       ;
      => dbCreateIndex(                                                 ;
                        <(file)>, <"key">, <{key}>,                     ;
                        if( <.u.>, .t., NIL )                           ;
                      ) ;                                               ;
         NtxCloseLog() ;                                                ;
         NtxLog( <(file)> )

// NOTE:  CLOSE <alias> must precede the others
#command CLOSE <alias>          => <alias>->( dbCloseArea(), DbfCloseLog() )

#command CLOSE                  => dbCloseArea() ; DbfCloseLog()
#command CLOSE DATABASES        => dbCloseAll()  ; DbfKillLog()
#command CLOSE INDEXES          => dbClearIndex() ; NtxCloseLog()



