#Include "FileLog.ch"     // #Include to allow for file logging.
#include "Inkey.ch"
#Include "Box.ch"

#Define CRLF        Chr(13) + Chr(10)

Function Main(cDbfName, cNtxName)
Local ScrSave := SaveSCreen(0, 0, MaxRow(), MaxCol())
Local OldRow := Row(), OldCol := Col()

    If cDbfName == NIL .or. cNtxName == NIL
        OutStd(CRLF+"NTXDEMO  -  Demo of the NTXRECNO function.  By John Van Ostrand."+CRLF)
        OutStd("    Usage: NTXDEMO <DbfName> <NtxName>"+CRLF+CRLF)
        Return NIL
    EndIf

    Use (cDbfName) Index (cNtxName)

    If ValType(&(IndexKey())) != "C"
        OutStd(CRLF+"NTXDEMO works better with a character index value."+CRLF+CRLF)
        Return NIL
    EndIf

        // Call the DBF browser.
    FileBrowse()

    RestScreen(0, 0, MaxRow(), MaxCol(), ScrSave)
    SetPos(OldRow, OldCol)
Return NIL




Function FileBrowse()
Local sBar, oTb, nKey, cSeekStr := ""
Local bNtxBlock := &("{||"+IndexKey()+"}")
Local ExprLen := Len(EVal(bNtxBlock))
Local TbWidth := ExprLen + 2, TbHeight
Local nLeft, nRight
Local LogRecNo := 1, NumRecs, cNtxName := Ntx() +".NTX"
Local OldRec

    nRight := (nLeft := (MaxCol() - TbWidth)/2) + TbWidth - 1

    DbGoBottom()

        // NTXRECNO() can be used to tell us how many records exist
        // in a DBF or for a 'while' condition.
    NumRecs := NTXRECNO(cNtxName, RecNo())
    DbGoTop()
    TbHeight := Min(NumRecs, 20)  // Determine the height of the browse window.

        // Setup the TBrowse object.
        // I will supply the skipBlock, goTop and Etc. functions
        // So I can increment the logical record number myself.
    SetColor("W+/B, W/R, , , W/R")
    oTb := TBrowseNew(4, nLeft, TbHeight + 3, nRight)
    oTb:AddColumn(TbColumnNew("Index Expression", bNtxBlock))
    oTb:GoTopBlock := { || LogRecNo := 1, DbGoTop() }
    oTb:GoBottomBlock := { || LogRecNo := NumRecs, DbGoBottom() }

        // We use skipblock to update the logical record number for us
        // only under normal circumstances.
    oTb:SkipBlock := {|n| Skipper(n, @LogRecNo) }
    oTb:HeadSep := ""

        // Draw our box.
    @ 3, nLeft - 2, TbHeight + 4, nRight + 2 Box B_DOUBLE + " "
    @ 0, 0, MaxRow(), MaxCol() Box "         " Color "W+/R"
    @ 0, 0 Say "NTXDEMO.EXE : Use Cursor Keys or type the desired record." Color "W+/R"
    @ 1, 0 Say "Demo of NTXRECNO(). We can now have a scroll bar on an indexed list." Color "W+/R"
    @ 2, 0 Say "Or use NTXRECNO() to show the progress of reports or other file processes." Color "W+/R"

        // Make and display the scroll bar.
    sBar := MakeBar(oTb:nTop, oTb:nRight + 2, oTb:nBottom, TbHeight)
    DisplayBar(sBar)
    UpdateBar(sBar, 1, NumRecs, .T.)


    nKey := 0
    While nKey != K_ESC .and. nKey != K_ENTER

            // Draw our TBrowse.
        While !oTb:Stabilize() .and. NextKey() == 0
        EndDo

            // Update the scroll bar indicator.
        UpdateBar(sBar, LogRecNo - oTb:RowPos + 1, NumRecs, .F.)

            // Print the typed characters.
        If oTb:Stable
            @ Row(), Col() Say cSeekStr Color "W+/N"
        EndIf

        nKey := Inkey(0)

        Do Case
        Case nKey == K_UP         // Standard TBrowse key handling
            oTb:Up()              // except we must reset cSeekStr
            cSeekStr := ""        // because were too lazy to see
        Case nKey == K_DOWN       // if it applies to the current
            oTb:Down()            // record.
            cSeekStr := ""
        Case nKey == K_PGUP
            oTb:PageUp()
            cSeekStr := ""
        Case nKey == K_PGDN
            oTb:PageDown()
            cSeekStr := ""
        Case nKey == K_HOME
            oTb:GoTop()
            cSeekStr := ""
        Case nKey == K_END
            oTb:GoBottom()
            cSeekStr := ""

        Case nKey == K_BS
                // Handle a BaskSpace character.
            If Len(cSeekStr) > 0
                cSeekStr := Left(cSeekStr, Len(cSeekStr) - 1)
                DbSeek(cSeekStr)

                    // Because we SEEK we must rely on NTXRECNO() to
                    // provide the logical record number.
                LogRecNo := NTXRECNO(cNtxName, RecNo())
                oTb:RefreshAll()
            EndIf
        Otherwise
                // Do a 'Seek-as-you-type'
            If nKey >= 32 .and. nKey <= 127

                // Save the current record in case wwe don't find a match.
                OldRec := RecNo()

                    // Checkfor a match.
                If DbSeek(cSeekStr + Upper(Chr(nKey)))

                        // A match is found so update LogRecNo and refresh.
                        // Because we SEEK we must rely on NTXRECNO() to
                        // provide the logical record number.
                    LogRecNo := NTXRECNO(cNtxName, RecNo())
                    cSeekStr += Upper(Chr(nKey))
                    oTb:RefreshAll()
                Else
                        // Not a valid character, beep !
                    Tone(300, .5)
                    DbGoto(OldRec)
                EndIf
            EndIf
        EndCase
    EndDo
Return NIL


/*
* Standard TBrowse skipper function except that we update
* the logical record number manually instead of using NTXRECNO().
* This is easier and more efficient.
*/

Static Function Skipper(n, LogRecNo)
Local i := 0

    Do Case
    Case n == 0
        DbSkip(0)
    Case n > 0
        While !Eof() .and. i < n
            DbSkip(1)
            i++
        EndDo
        If Eof()
            DbSkip(-1)
            i--
        EndIf
    Case n < 0
        While !Bof() .and. i > n
            DbSkip(-1)
            i--
        EndDo
        If Bof()
            DbGoTop()
            i++
        EndIf
    EndCase

        // update the logical record number.
    LogRecNo += i
Return ( i )

